/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// browse.cpp                                                              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.34 - 11 mar 1995                       //
// Author: H. Picot                                                        //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////


#include "griddll.h"
#pragma hdrstop
#include "browse.h"
#include <errno.h>
#include <owl/opensave.h>

// OwlMain                                                                *
int OwlMain (int, char *[])
   {
   if (::MessageBox (0,
      "This program is a VERY crude dbf file browser,\r\n"
      "whose sole purpose is to illustrate how to use\r\n"
      "TGrid in a dynamically created window.\r\n\n"
      "****** DON'T USE IT ON VALUABLE DATA ******",
      "!!! WARNING !!!",
      MB_OKCANCEL|MB_DEFBUTTON2) == IDCANCEL)
      return 0;

   try {
      return Application().Run();
      }
   catch (FILE *fd)
      {
      if (fd == 0)
         ::MessageBox (0, _sys_errlist[errno], "Dbf Browser", IDOK);
      else
         {
         fclose (fd);
         ::MessageBox (0, "No records in file", "Dbf Browser", IDOK);
         }
      }
   catch (char *s)
      {
      ::MessageBox (0, s, "Dbf Browser", IDOK);
      }
   catch (...)
      {
      }
   return 0;
   }


// class Application
Application::Application () : TApplication ()
   {
   }

   
void Application::InitMainWindow ()
   {
   EnableCtl3d (true);
   SetMainWindow (new TFrameWindow (0, "Dbf browser", new DbfBrowser ()));
   }


// class TGridStringField
class TGridStringField : public TGEdit
   {
   protected:
      void PerformCreate (int idOrMenu);

   public:
      TGridStringField (TWindow *p, int id, int w, int h, int fw);
   };

TGridStringField::TGridStringField (TWindow *p, int id, int w, int h, int fw)
                 : TGEdit (p, id, "", 0, 0, w, h, fw)
   {
   }

void TGridStringField::PerformCreate (int idOrMenu)
   {
   Attr.Style = ES_LEFT|ES_AUTOHSCROLL|WS_TABSTOP|WS_BORDER|WS_CHILD|ES_MULTILINE;
   TGEdit::PerformCreate (idOrMenu);
   }

// class TGridDateField
class TGridDateField : public TGDateEdit
   {
   protected:
      void PerformCreate (int idOrMenu);

   public:
      TGridDateField (TWindow *p, int id, int w, int h, string &fmt, IntlDate &min, IntlDate &max);
   };

TGridDateField::TGridDateField (TWindow *p, int id, int w, int h,
                                string &fmt, IntlDate &min, IntlDate &max)
                 : TGDateEdit (p, id, 0, 0, w, h, fmt, min, max, false)
   {
   }

void TGridDateField::PerformCreate (int idOrMenu)
   {
   Attr.Style = ES_LEFT|ES_AUTOHSCROLL|WS_TABSTOP|WS_BORDER|WS_CHILD|ES_MULTILINE;
   TGEdit::PerformCreate (idOrMenu);
   }


// class TGridNumericField
class TGridNumericField : public TGDoubleEdit
   {
   protected:
      void PerformCreate (int idOrMenu);

   public:
      TGridNumericField (TWindow *p, int id, int w, int h,
                         int width, int dec, double min, double max);
   };

TGridNumericField::TGridNumericField (TWindow *p, int id, int w, int h,
                                int width, int dec, double min, double max)
                 : TGDoubleEdit (p, id, 0.0, 0, 0, w, h, width, dec, min, max)
   {
   }

void TGridNumericField::PerformCreate (int idOrMenu)
   {
   Attr.Style = ES_RIGHT|ES_AUTOHSCROLL|WS_TABSTOP|WS_BORDER|WS_CHILD|ES_MULTILINE;
   TGEdit::PerformCreate (idOrMenu);
   }


// class DbfBrowser
DEFINE_RESPONSE_TABLE1 (DbfBrowser, TWindow)
   EV_WM_WINDOWPOSCHANGED,
END_RESPONSE_TABLE;

DbfBrowser::DbfBrowser () : TWindow (0, 0, 0)
   {
   grid = new BrowserGrid (this);
   }


void DbfBrowser::EvWindowPosChanged (WINDOWPOS far &wpos)
   {
   TWindow::EvWindowPosChanged (wpos);
   TRect rc (GetClientRect ());
   grid->SetWindowPos (0, rc, SWP_NOZORDER);
   }


// class BrowserGrid
BrowserGrid::BrowserGrid (TWindow *p)
           : TGrid (p, 100, 0, 0, 0, 0,
                    TGrid::NextLine,
                    TGrid::TabNextColumn,
                    TGrid::ReverseColor,
                    -1,
                    TGrid::gAllBorders 
                  | TGrid::gHideHScroll
                  | TGrid::gCenterTitle
                  | TGrid::gLineSwapOn
                  | TGrid::gSizeColumn )
   {
   TOpenSaveDialog::TData data (OFN_FILEMUSTEXIST|OFN_HIDEREADONLY,
             "DBF files (*.dbf)|*.DBF|All files (*.*)|*.*|", 0, 0, "DBF");

   TFileOpenDialog *fod = new TFileOpenDialog (GetApplication()->GetMainWindow(), data, 0, "Sample DBF browser");
   if (fod->DoExecute () != IDOK)
      throw ("Abort");
   delete fod;

   fd = fopen (data.FileName, "rb+");
   if (fd == 0)
      throw (fd);

   InitFields ();

   for (int i = 0; i < nfields; i++)
      Add (fields[i].ctrl, fields[i].name, fields[i].align);

   SetVBarText (8);  // make vbar 8 char wide, text is user-defined
   }


void BrowserGrid::CalcPos ()
   {
   TGrid::CalcPos ();
   }
   

BrowserGrid::~BrowserGrid ()
   {
   if (fd)
      fclose (fd);
   }


const void *BrowserGrid::TransferItem (int line, int id)
   {
   if ((unsigned)line >= (unsigned)nrec)
      return 0;

   long pos = (long(line) * size) + offset;

   static char buffer[256];
   memset (buffer, 0, sizeof(buffer));

   switch (id)
      {
      case 0:
         {
         char del;
         fseek (fd, pos, 0);
         fread (&del, 1, 1, fd);
         sprintf (buffer, "%d %c", line+1, del);
         break;
         }
      default:
         {
         id -= 1;
         Field &fld = fields[id];
         fseek (fd, pos+fld.offset, 0);
         fread (buffer, fld.width, 1, fd);
         buffer[fld.width] = 0;
         if (fld.type == 'N' || fld.type == 'F')
            {
            double f = atof (buffer);
            memcpy (buffer, &f, sizeof(f));
            }
         else if (fld.type == 'D')
            {
            IntlDate date (buffer, "YYYYMMDD");
            memcpy (buffer, &date, sizeof(date));
            }
         break;
         }
      }

   return static_cast<void *>(buffer);
   }


void BrowserGrid::TransferItem (int line, int id, const void *data)
   {
   if (id && data)
      {
      id -= 1;
      long pos = (long(line) * size) + offset;
      Field &fld = fields[id];
      fseek (fd, pos+fld.offset, 0);
      if (fld.type == 'N' || fld.type == 'F')
         {
         double f = *static_cast<const double *>(data);
         char buffer[255];
         sprintf (buffer, "%*.*f", fld.width, fld.decimals, f);
         if (strlen (buffer) > fld.width)
            memset (buffer, '*', fld.width);
         fwrite (buffer, fld.width, 1, fd);
         }
      else if (fld.type == 'D')
         {
         IntlDate date = *static_cast<const IntlDate *>(data);
         string s = date.String ("YYYYMMDD");
         fwrite (s.c_str(), 8, 1, fd);
         }
      else
         fwrite (data, fld.width, 1, fd);
      }
   }

long BrowserGrid::InitFields ()
   {
   fseek (fd, 4L, 0);
   fread (&nrec, 4, 1, fd);
   if (nrec == 0L)
      throw (fd);
   fread (&offset, 2, 1, fd);
   fread (&size, 2, 1, fd);
   nfields = (offset-33)/32;
   fields = new Field[nfields];

   int w = LOWORD(GetDialogBaseUnits());
   int h = HIWORD(GetDialogBaseUnits()) + 5;
   int off = 1;  // field offset

   string datefmt = GetLanguage().SysShortDate ();
   IntlDate min (1,1,1750);
   IntlDate max (31,12,9999);

   long pos = 32;
   for (int i = 0; i < nfields; i++, pos += 32)
      {
      Field &fld = fields[i];
      fseek (fd, pos, 0);
      fread (fld.name, 10, 1, fd);
      fld.name[10] = 0;

      fseek (fd, pos+11, 0);
      fread (&fld.type, 1, 1, fd);

      fseek (fd, pos+16, 0);
      unsigned char width, decimals;
      fread (&width, 1, 1, fd);
      fread (&decimals, 1, 1, fd);
      fld.width = width;
      fld.decimals = decimals;
      fld.align = TGrid::Left;
      fld.offset = off;
      off += width;

      int l = strlen(fld.name);
      if (width < l)
         width = (short)l;

      int cx = ((width*w*4)/3)+5;   // average width
      int id = 1 + i;
      switch (fld.type)
         {
         case 'M':
         case 'G':
            fld.ctrl = new TGStatic (this, id, "", 0, 0, cx, h, fld.width+1);
            break;

         case 'D':
            fld.ctrl = new TGridDateField (this, id, cx, h, datefmt, min, max);
            break;

         case 'N': case 'F':
            {
            double fmin, fmax;
            char buffer[255];
            memset (buffer, '9', fld.width);
            buffer[fld.width] = 0;
            if (fld.decimals)
               buffer[fld.width-fld.decimals-1] = '.';
            fmax = atof(buffer);
            if (buffer[1] != '.')
               buffer[0] = '-';
            else
               strcpy (buffer, "0");
            fmin = atof(buffer);
            fld.ctrl = new TGridNumericField (this, id, cx, h, fld.width, fld.decimals, fmin, fmax);
            fld.align = TGrid::Right;
            break;
            }

         default:
            fld.ctrl = new TGridStringField (this, id, cx, h, fld.width+1);
            break;
         }
      }

   return nrec;
   }

bool BrowserGrid::AppendNewLine (int &newline)
   {
   if ((unsigned)newline >= (unsigned)nrec)
      return false;
   else
      return true;
   }


// EOF
